'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _nesthydrationjs = require('nesthydrationjs');

var _queryASTToSqlAST = require('./queryASTToSqlAST');

var _queryASTToSqlAST2 = _interopRequireDefault(_queryASTToSqlAST);

var _stringifySqlAST = require('./stringifySqlAST');

var _stringifySqlAST2 = _interopRequireDefault(_stringifySqlAST);

var _makeNestHydrationSpec = require('./makeNestHydrationSpec');

var _makeNestHydrationSpec2 = _interopRequireDefault(_makeNestHydrationSpec);

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('join-monster');

function joinMonster(ast, context, dbCall) {
  const sqlAST = (0, _queryASTToSqlAST2.default)(ast);
  debug((0, _util.emphasize)('SQL_AST'), (0, _util.inspect)(sqlAST));

  const sql = (0, _stringifySqlAST2.default)(sqlAST, context);
  debug((0, _util.emphasize)('SQL'), (0, _util.inspect)(sql));
  if (!sql) return Promise.resolve({});

  const nestSpec = (0, _makeNestHydrationSpec2.default)(sqlAST);
  debug((0, _util.emphasize)('NEST_SPEC'), (0, _util.inspect)(nestSpec));

  if (dbCall.length === 2) {
    return new Promise((resolve, reject) => {
      dbCall(sql, (err, rows) => {
        if (err) {
          reject(err);
        } else {
          debug((0, _util.emphasize)('RAW_DATA'), (0, _util.inspect)(rows));
          resolve((0, _nesthydrationjs.nest)(rows, nestSpec));
        }
      });
    });
  }

  const result = dbCall(sql);

  if (result.then) {
    return result.then(rows => {
      debug((0, _util.emphasize)('RAW DATA'), (0, _util.inspect)(rows));
      return (0, _nesthydrationjs.nest)(rows, nestSpec);
    });
  } else {
    return Promise.resolve((0, _nesthydrationjs.nest)(result, nestSpec));
  }
}

joinMonster.version = require('../package.json').version;

exports.default = joinMonster;