'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = makeNestHydrationSpec;

var _util = require('./util');

function makeNestHydrationSpec(topNode) {
  (0, _util.validateSqlAST)(topNode);
  return _makeNestHydrationSpecForSqlAST(null, '', topNode);
}

function _makeNestHydrationSpecForSqlAST(parent, prefix, node) {
  if (node.table) {
    const prefixToPass = parent ? prefix + node.as + '__' : prefix;

    let columnDeps = new Set();
    for (let c of node.children) {
      var _context;

      if (c.columnDeps) c.columnDeps.forEach((_context = columnDeps).add.bind(_context));
    }
    columnDeps = Array.from(columnDeps);

    const columns = node.children.filter(c => c.column);
    const tables = node.children.filter(c => c.table);

    const fieldDefinition = {};
    columnDeps.forEach(col => fieldDefinition[col] = prefixToPass + col);
    columns.forEach(col => fieldDefinition[col.fieldName] = prefixToPass + col.column);

    for (let table of tables) {
      const spec = _makeNestHydrationSpecForSqlAST(node, prefixToPass, table);
      fieldDefinition[table.fieldName] = spec;
    }

    if (node.grabMany) {
      return [fieldDefinition];
    } else {
      return fieldDefinition;
    }
  } else {
    throw new Error('unexpected/unknown node type reached: ' + (0, _util.inspect)(node));
  }
}