'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = queryASTToSqlAST;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function queryASTToSqlAST(ast) {
  const usedTableAliases = new Set();

  const sqlAST = {};
  _assert2.default.equal(ast.fieldASTs.length, 1, 'We thought this would always have a length of 1. FIX ME!!');

  const queryAST = ast.fieldASTs[0];

  const parentType = ast.parentType;
  getGraphQLType(queryAST, parentType, sqlAST);
  return sqlAST;

  function getGraphQLType(queryASTNode, parentTypeNode, sqlASTNode) {
    const fieldName = queryASTNode.name.value;

    const field = parentTypeNode._fields[fieldName];

    let grabMany = false;

    let gqlType = stripNonNullType(field.type);

    if (gqlType.constructor.name === 'GraphQLList') {
      gqlType = gqlType.ofType;
      grabMany = true;
    }

    const config = gqlType._typeConfig;

    if (gqlType.constructor.name === 'GraphQLObjectType' && config.sqlTable) {
      sqlASTNode.table = config.sqlTable;
      if (!config.sqlTable) {
        throw new Error(`Must specify "sqlTable" property on ${ field.type.name } GraphQLObjectType definition.`);
      }

      sqlASTNode.as = makeUnique(usedTableAliases, field.name);

      if (queryASTNode.arguments.length) {
        const args = sqlASTNode.args = {};
        for (let arg of queryASTNode.arguments) {
          args[arg.name.value] = arg.value.value;
        }
      }

      sqlASTNode.fieldName = field.name;
      sqlASTNode.grabMany = grabMany;

      if (field.where) {
        sqlASTNode.where = field.where;
      }
      if (field.sqlJoin) {
        sqlASTNode.sqlJoin = field.sqlJoin;
      }
      if (field.joinTable) {
        sqlASTNode.sqlJoins = field.sqlJoins;
        sqlASTNode.joinTable = field.joinTable;
        sqlASTNode.joinTableAs = makeUnique(usedTableAliases, field.joinTable);
      }

      sqlASTNode.children = [];

      if (grabMany) {
        if (!config.uniqueKey) {
          throw new Error(`Requesting a list of ${ config.sqlTable }. You must specify the "uniqueKey" on the GraphQLObjectType definition`);
        }
        sqlASTNode.children.push({ column: config.uniqueKey, fieldName: config.uniqueKey });
      }
      if (queryASTNode.selectionSet) {
        for (let selection of queryASTNode.selectionSet.selections) {
          switch (selection.kind) {
            case 'Field':
              growNewTreeAndAddToChildren(sqlASTNode.children, selection, gqlType);
              break;

            case 'InlineFragment':
              for (let fragSelection of selection.selectionSet.selections) {
                growNewTreeAndAddToChildren(sqlASTNode.children, fragSelection, gqlType);
              }
              break;

            case 'FragmentSpread':
              const fragmentName = selection.name.value;
              const fragment = ast.fragments[fragmentName];
              for (let fragSelection of fragment.selectionSet.selections) {
                growNewTreeAndAddToChildren(sqlASTNode.children, fragSelection, gqlType);
              }
              break;
            default:
              throw new Error('Unknown selection kind: ' + selection.kind);
          }
        }
      }
    } else if (field.sqlColumn || !field.resolve) {
      sqlASTNode.column = field.sqlColumn || field.name;
      sqlASTNode.fieldName = field.name;
    } else if (field.sqlDeps) {
      sqlASTNode.columnDeps = field.sqlDeps;
    } else {
      sqlASTNode.noop = true;
    }
  }

  function growNewTreeAndAddToChildren(children, selection, graphQLType) {
    const newNode = {};
    children.push(newNode);
    getGraphQLType(selection, graphQLType, newNode);
  }
}

function stripNonNullType(type) {
  return type.constructor.name === 'GraphQLNonNull' ? type.ofType : type;
}

function makeUnique(usedNames, name) {
  if (usedNames.has(name)) {
    name += '$';
  }
  usedNames.add(name);
  return name;
}