'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = stringifySqlAST;

var _util = require('./util');

function stringifySqlAST(topNode, context) {
  (0, _util.validateSqlAST)(topNode);
  let { selections, joins, wheres } = _stringifySqlAST(null, topNode, '', context, [], [], []);

  selections = [...new Set(selections)];
  if (!selections.length) return '';
  return 'SELECT\n  ' + selections.join(',\n  ') + '\n' + joins.join('\n') + '\n' + wheres.join('\n');
}

function _stringifySqlAST(parent, node, prefix, context, selections, joins, wheres) {
  if (node.table) {
    if (node.where) {
      const whereCondition = node.where(`"${ node.as }"`, node.args || {}, context);
      if (whereCondition) {
        wheres.push(`WHERE ${ whereCondition }`);
      }
    }

    if (node.sqlJoin) {
      const joinCondition = node.sqlJoin(`"${ parent.as }"`, `"${ node.as }"`);

      joins.push(`LEFT JOIN "${ node.table }" AS "${ node.as }" ON ${ joinCondition }`);
    } else if (node.joinTable) {
      if (!node.sqlJoins) throw new Error('Must set "sqlJoins" for a join table.');
      const joinCondition1 = node.sqlJoins[0](`"${ parent.as }"`, `"${ node.joinTableAs }"`);
      const joinCondition2 = node.sqlJoins[1](`"${ node.joinTableAs }"`, `"${ node.as }"`);

      joins.push(`LEFT JOIN "${ node.joinTable }" AS "${ node.joinTableAs }" ON ${ joinCondition1 }`, `LEFT JOIN "${ node.table }" AS "${ node.as }" ON ${ joinCondition2 }`);
    } else {
      joins.push(`FROM "${ node.table }" AS "${ node.fieldName }"`);
    }

    for (let child of node.children) {
      _stringifySqlAST(node, child, parent ? prefix + node.as + '__' : prefix, context, selections, joins, wheres);
    }
  } else if (node.column) {
    selections.push(`"${ parent.as }"."${ node.column }" AS "${ prefix + node.column }"`);
  } else if (node.columnDeps) {
    node.columnDeps.forEach(col => selections.push(`"${ parent.as }"."${ col }" AS "${ prefix + col }"`));
  } else if (node.noop) {
    return;
  } else {
    throw new Error('unexpected/unknown node type reached: ' + (0, _util.inspect)(node));
  }
  return { selections, joins, wheres };
}